package cn.iocoder.yudao.module.ecw.service.api;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.module.ecw.api.bank.BankApi;
import cn.iocoder.yudao.module.ecw.api.bank.dto.BankAccountDTO;
import cn.iocoder.yudao.module.ecw.convert.bankAccount.BankAccountConvert;
import cn.iocoder.yudao.module.ecw.dal.dataobject.bankAccount.BankAccountDO;
import cn.iocoder.yudao.module.ecw.dal.mysql.bankAccount.BankAccountMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

@Service
public class BankApiImpl implements BankApi {
    @Resource
    private BankAccountMapper bankAccountMapper;

    @Override
    public List<BankAccountDTO> getBankAccountByAccountNo(String baAccountNum) {
        List<BankAccountDO> bankAccountDOs = bankAccountMapper.selectList("ba_account_num", baAccountNum);
        return BankAccountConvert.INSTANCE.convertDTOList(bankAccountDOs);
    }

    @Override
    public BankAccountDTO getBankAccountByAccountId(Long id) {
        BankAccountDO bankAccountDO = bankAccountMapper.selectById(id);
        return BankAccountConvert.INSTANCE.convertDTO(bankAccountDO);
    }

    @Override
    public void updateBankAccountBalance(BankAccountDTO bankAccountDTO, boolean isAdd) {
        BankAccountDO bankAccountDO = null;
        if (bankAccountDTO.getId() != null) {
            bankAccountDO = bankAccountMapper.selectById(bankAccountDTO.getId());
        } else {
            List<BankAccountDO> bankAccountDOS = bankAccountMapper.selectList(Wrappers.lambdaQuery(BankAccountDO.class)
                    .eq(BankAccountDO::getBaAccountName, bankAccountDTO.getBaAccountName())
                    .eq(BankAccountDO::getBaAccountNum, bankAccountDTO.getBaAccountNum())
                    .eq(BankAccountDO::getBaBankName, bankAccountDTO.getBaBankName()));
            if (CollectionUtil.isNotEmpty(bankAccountDOS)) {
                bankAccountDO = bankAccountDOS.get(0);
            }
        }
        if (bankAccountDO != null) {
            BigDecimal baBalance = bankAccountDO.getBaBalance() == null ? BigDecimal.ZERO : bankAccountDO.getBaBalance();
            if (isAdd) {
                bankAccountDO.setBaBalance(baBalance.add(bankAccountDTO.getBaBalance()));
            } else {
                bankAccountDO.setBaBalance(baBalance.subtract(bankAccountDTO.getBaBalance()));
            }
            bankAccountMapper.updateById(bankAccountDO);
        }
    }
}
