package cn.iocoder.yudao.module.ecw.service.api;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.convert.currency.CurrencyConvert;
import cn.iocoder.yudao.module.ecw.convert.exchangeRate.ExchangeRateConvert;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.exchangeRate.ExchangeRateDO;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.ecw.service.exchangeRate.ExchangeRateService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.EXCHANGE_RATE_EXPIRE;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.EXCHANGE_RATE_NOT_EXISTS_PARAM;

@Service
public class CurrecyApiImpl implements CurrencyApi {

    @Resource
    CurrencyService currencyService;

    @Resource
    ExchangeRateService exchangeRateService;

    @Override
    public Map<Integer, CurrencyRespDTO> getAllCurrency() {
        return currencyService.getCurrencyList().stream().collect(Collectors.toMap(CurrencyDO::getId, t -> CurrencyConvert.INSTANCE.convert2(t)));
    }

    @Override
    public ExchangeRateRespDTO getCurrencyRate(Long sourceId, Long targetId) {
        if (sourceId == targetId) {
            ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
            dto.setCurrencyRate(new BigDecimal("1"));
            dto.setSourceCurrencyId(sourceId);
            dto.setTargetCurrencyId(targetId);
            return dto;
        }
        LambdaQueryWrapper<ExchangeRateDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ExchangeRateDO::getSourceCurrencyId, sourceId);
        lambdaQueryWrapper.eq(ExchangeRateDO::getTargetCurrencyId, targetId);
        ExchangeRateDO rateDO = exchangeRateService.getOne(lambdaQueryWrapper);

        CurrencyDO currencyDO1 = currencyService.getCurrency(sourceId.intValue());
        CurrencyDO currencyDO2 = currencyService.getCurrency(targetId.intValue());

        if (null != rateDO) {
            //判断汇率有没有过期
            if (null != rateDO.getExpireDate()) {
                if (DateUtil.compare(rateDO.getExpireDate(), DateUtil.date()) == -1) {
                    if (null == currencyDO1 || null == currencyDO2) {
                        throw exception(EXCHANGE_RATE_EXPIRE, sourceId.toString(), targetId.toString());
                    } else {
                        throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                    }
                }
            } else {
                if (null == currencyDO1 || null == currencyDO2) {
                    throw exception(EXCHANGE_RATE_EXPIRE, sourceId.toString(), targetId.toString());
                } else {
                    throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                }
            }
            return ExchangeRateConvert.INSTANCE.convert2(exchangeRateService.getOne(lambdaQueryWrapper));
        } else {
            if (null == currencyDO1 || null == currencyDO2) {
                ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
                dto.setCurrencyRate(BigDecimal.ZERO);
                dto.setSourceCurrencyId(sourceId);
                dto.setTargetCurrencyId(targetId);
                return dto;
            } else {
                throw exception(EXCHANGE_RATE_NOT_EXISTS_PARAM, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
            }
        }

    }

    @Override
    public List<ExchangeRateRespDTO> getCurrencyRateList(Collection<Long> sourceIds, Long targetId) {
        List<ExchangeRateRespDTO> exchangeRateRespDTOList = new ArrayList<>();
        for (Long sourceId : sourceIds) {
            if (sourceId == targetId) {
                ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
                dto.setCurrencyRate(new BigDecimal("1"));
                dto.setSourceCurrencyId(sourceId);
                dto.setTargetCurrencyId(targetId);
                exchangeRateRespDTOList.add(dto);
                continue;
            }
            LambdaQueryWrapper<ExchangeRateDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(ExchangeRateDO::getSourceCurrencyId, sourceId);
            lambdaQueryWrapper.eq(ExchangeRateDO::getTargetCurrencyId, targetId);
            ExchangeRateDO rateDO = exchangeRateService.getOne(lambdaQueryWrapper);

            CurrencyDO currencyDO1 = currencyService.getCurrency(sourceId.intValue());
            CurrencyDO currencyDO2 = currencyService.getCurrency(targetId.intValue());


            if (null != rateDO) {
                //判断汇率有没有过期
                if (null != rateDO.getExpireDate()) {
                    if (DateUtil.compare(rateDO.getExpireDate(), DateUtil.date()) == -1) {
                        if (null == currencyDO1 || null == currencyDO2) {
                            throw exception(EXCHANGE_RATE_EXPIRE, sourceId.toString(), targetId.toString());
                        } else {
                            throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                        }
                    }
                } else {
                    if (null == currencyDO1 || null == currencyDO2) {
                        throw exception(EXCHANGE_RATE_EXPIRE, sourceId.toString(), targetId.toString());
                    } else {
                        throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                    }
                }
                exchangeRateRespDTOList.add(ExchangeRateConvert.INSTANCE.convert2(rateDO));
            } else {
                if (null == currencyDO1 || null == currencyDO2) {
                    ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
                    dto.setCurrencyRate(BigDecimal.ZERO);
                    dto.setSourceCurrencyId(sourceId);
                    dto.setTargetCurrencyId(targetId);
                    exchangeRateRespDTOList.add(dto);
                } else {
                    throw exception(EXCHANGE_RATE_NOT_EXISTS_PARAM, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                }
            }
        }
        return exchangeRateRespDTOList;

    }

    @Override
    public ExchangeRateRespDTO getCurrencyRateByCode(String sourceCode, String targetCode) {
        if (StrUtil.equals(sourceCode, targetCode)) {
            ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
            dto.setCurrencyRate(new BigDecimal("1"));
            dto.setSourceCurrencyCode(sourceCode);
            dto.setTargetCurrencyCode(targetCode);
            return dto;
        }
        LambdaQueryWrapper<ExchangeRateDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ExchangeRateDO::getSourceCurrencyCode, sourceCode);
        lambdaQueryWrapper.eq(ExchangeRateDO::getTargetCurrencyCode, targetCode);
        ExchangeRateDO rateDO = exchangeRateService.getOne(lambdaQueryWrapper);

        CurrencyDO currencyDO1 = currencyService.getCurrencyByCode(sourceCode);

        CurrencyDO currencyDO2 = currencyService.getCurrencyByCode(targetCode);


        if (null != rateDO) {
            //判断汇率有没有过期
            if (null != rateDO.getExpireDate()) {
                if (DateUtil.compare(rateDO.getExpireDate(), DateUtil.date()) == -1) {
                    if (null == currencyDO1 || null == currencyDO2) {
                        throw exception(EXCHANGE_RATE_EXPIRE, sourceCode, targetCode);
                    } else {
                        throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                    }
                }
            } else {
                if (null == currencyDO1 || null == currencyDO2) {
                    throw exception(EXCHANGE_RATE_EXPIRE, sourceCode, targetCode);
                } else {
                    throw exception(EXCHANGE_RATE_EXPIRE, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
                }
            }
            return ExchangeRateConvert.INSTANCE.convert2(exchangeRateService.getOne(lambdaQueryWrapper));
        } else {
            if (null == currencyDO1 || null == currencyDO2) {
                ExchangeRateRespDTO dto = new ExchangeRateRespDTO();
                dto.setCurrencyRate(BigDecimal.ZERO);
                dto.setSourceCurrencyCode(sourceCode);
                dto.setTargetCurrencyCode(targetCode);
                return dto;
            } else {
                throw exception(EXCHANGE_RATE_NOT_EXISTS_PARAM, currencyDO1.getTitleZh() + currencyDO1.getFuhao(), currencyDO2.getTitleZh() + currencyDO2.getFuhao());
            }
        }


    }

    @Override
    public List<ExchangeRateRespDTO> getAllCurrencyRate() {
        List<ExchangeRateDO> exchangeRateDOS = exchangeRateService.selectList();
        return ExchangeRateConvert.INSTANCE.convertList2(exchangeRateDOS);
    }
}
