package cn.iocoder.yudao.module.ecw.service.bankAccount;

import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.dal.mysql.region.RegionMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;

import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.module.ecw.controller.admin.bankAccount.vo.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.bankAccount.BankAccountDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.ecw.convert.bankAccount.BankAccountConvert;
import cn.iocoder.yudao.module.ecw.dal.mysql.bankAccount.BankAccountMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 银行账户 Service 实现类
 *
 * @author 久点源码
 */
@Service
@Validated
public class BankAccountServiceImpl implements BankAccountService {

    @Resource
    private BankAccountMapper bankAccountMapper;
    @Resource
    private RegionMapper regionMapper;

    @Override
    public Long createBankAccount(BankAccountCreateReqVO createReqVO) {
        // 插入
        BankAccountDO bankAccount = BankAccountConvert.INSTANCE.convert(createReqVO);
//        bankAccount.setSource(createReqVO.getSource());
        bankAccountMapper.insert(bankAccount);
        // 返回
        return bankAccount.getId();
    }

    @Override
    public void updateBankAccount(BankAccountUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBankAccountExists(updateReqVO.getId());
        // 更新
        BankAccountDO updateObj = BankAccountConvert.INSTANCE.convert(updateReqVO);
//        updateObj.setSource(updateReqVO.getSource());
        bankAccountMapper.updateById(updateObj);
    }

    @Override
    public void updateBankAccountStatus(BankAccountUpdateStatusReqVO updateReqVO) {
        // 校验存在
        this.validateBankAccountExists(updateReqVO.getId());
        // 更新
        BankAccountDO updateObj = BankAccountConvert.INSTANCE.convert(updateReqVO);
        bankAccountMapper.updateById(updateObj);
    }

    @Override
    public void deleteBankAccount(Long id) {
        // 校验存在
        this.validateBankAccountExists(id);
        // 删除
        bankAccountMapper.deleteById(id);
    }

    private void validateBankAccountExists(Long id) {
        if (bankAccountMapper.selectById(id) == null) {
            throw exception(BANK_ACCOUNT_NOT_EXISTS);
        }
    }

    @Override
    public BankAccountDO getBankAccount(Long id) {
        return bankAccountMapper.selectById(id);
    }

    @Override
    public List<BankAccountDO> getBankAccountList(Collection<Long> ids) {
        return bankAccountMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BankAccountRespVO> getBankAccountPage(BankAccountPageReqVO pageReqVO) {
        PageResult<BankAccountDO> bankAccountDOPageResult = bankAccountMapper.selectPage(pageReqVO);
        PageResult<BankAccountRespVO> voPageResult = BankAccountConvert.INSTANCE.convertPage(bankAccountDOPageResult);
        List<RegionDO> countryList = regionMapper.getImportExportCountryList();

        Map<Long, RegionDO> countryMap = countryList.stream().collect(Collectors.toMap(RegionDO::getId, regionDO -> regionDO));
        voPageResult.getList().forEach(vo -> {
            if (vo.getBaCountry() != null) {
                RegionDO baCountryDO = countryMap.get(vo.getBaCountry().longValue());
                vo.setBaCountryZh(baCountryDO.getTitleZh());
                vo.setBaCountryEn(baCountryDO.getTitleEn());
            }
        });
        return voPageResult;
    }

    @Override
    public List<BankAccountDO> getBankAccountList(BankAccountExportReqVO exportReqVO) {
        return bankAccountMapper.selectList(exportReqVO);
    }

}
